<?php
/**
  * @author Auguria <info@auguria.net>
  * @copyright Auguria
  * @license http://www.opensource.org/licenses/osl-3.0.php Open-source licence 3.0
  * @version 1.0
  */

if(!defined('_PS_ROOT_DIR_'))
{
	include_once(dirname(__FILE__).'/dranersms_api.php');
}
else
{
	include_once(_PS_ROOT_DIR_ . '/modules/dranersms/dranersms_api.php');
}

class DranerSms extends Module {
	private $_html  = '';
	private $_postErrors;
	private $_balance = null;
	
	/* ************************************ *
	 * STANDARD PRESTASHOP MODULE FUNCTIONS *
	 * ************************************ */
	function __construct() {
		$this->name = 'dranersms';
		
		parent::__construct();
		
		$this->displayName = '123 SMS';
		$this->description = $this->l('Sends notifications using 123 Sms platform');
		$this->tab = 'Tools';
		$this->version = 1.0;
		
	}
	
	/**
	 * Installation : register hooks and default configuration values
	 */
	public function install() {
		if 	(	!parent::install() 
				OR !$this->registerHook('newOrder')
				OR !$this->registerHook('updateOrderStatus')
				OR !ConfigurationCore::updateValue('DranerSms_STATUS_SELECTED', '[]')
			)
			return false;
		
		foreach ($this->_getAdminParameters() as $p) {
			if(key_exists('name',$p)) {
				if(!Configuration::updateValue($p['name'],$p['default']))
					return false;
			}
		}
		return true;
	}

	
	/**
	 * Unregister configuration values
	 */
	public function uninstall()
	{
		ConfigurationCore::deleteByName('DranerSms_STATUS_SELECTED');
		foreach ($this->_getAdminParameters() as $p) {
			if(key_exists('name',$p)) {
				if(!Configuration::deleteByName($p['name']))
					return false;
			}
		}
		return parent::uninstall();
	}
	
	/* ******************* *
	 * ADMINISTRATION FORM *
	 * ******************* */
	/**
	 * Return administration form
	 */
	public function getContent() {
		if(Tools::isSubmit('DranerSmsupdate') || Tools::isSubmit('DranerSmstest')) {
			$this->_postValidation();			
			if (!sizeof($this->_postErrors)) {
				// Valid form, update values
				foreach ($this->_getAdminParameters() as $p) {
					if($p['type']=='checkbox') {
						$value = Tools::getIsset($p['name']) ? 1 : 0;
						Configuration::updateValue($p['name'], $value);
					}else if(key_exists('name',$p) && $p['type'] != 'static') {
						Configuration::updateValue($p['name'],Tools::getValue($p['name']));
					}
				}
				$this->_html .= '<div class="conf confirm">'.$this->l('Settings updated').'</div>';				
				if(Tools::isSubmit('DranerSmstest')) {
					if($this->_sendSMS($this->l('DranerSms for prestashop configuration test'),array()))
						$this->_html .= '<div class="conf confirm">'.$this->l('A test sms has been sent to your phone').'</div>';
					else
						$this->_html .= '<div class="alert error">'.$this->l('An error occured when sending a test sms').'</div>';
				}
			}
			else {
				// Display form error
				foreach ($this->_postErrors as $err) {
					$this->_html .= '<div class="alert error">'.$err.'</div>';
				}
			}
		}
		
		$this->_displayForm();
		
		return $this->_html;
	}
	
	/**
	 * Check inputs
	 * @return array the list of errors found
	 */
	private function _postValidation() {
		$this->_postErrors = array();
		
		foreach($this->_getAdminParameters() as $p) {
			if(array_key_exists('validate',$p)) {
				$validate_function = $p['validate'];
				$error = $this->$validate_function(Tools::getValue($p['name']));
				if($error != null)
					$this->_postErrors[] = $error;
			}
		}
	}
	
	private function _checkUser($input) {
		if(!preg_match('#^[A-Za-z0-9]{0,30}$#',$input) && !Validate::isEmail($input))
			return $this->l('Invalid user name');
		return null;
	}
	
	private function _checkPassword($input) {
		if(!preg_match('#^[A-Za-z0-9]{0,30}$#',$input))
		{
			return $this->l('Invalid password');
		}
		return null;
	}
	
	private function _checkPhone($input) {
		$error = null;
		$numbers = explode(';',$input);
		foreach($numbers as $num)
		{
			if(!Validate::isPhoneNumber($num))
				$error = $this->l('Invalid phone number(s)');
		}
		return $error;
	}
	
	private function _checkOrderText($input) {
		$tokens = array('{price}','{currency}','{payment}','{shopname}','{orderid}','{firstname}','{lastname}');
		$input = str_replace($tokens,'',$input);
		if(!DranerSms_API::isValidText($input))
			return $this->l('Invalid "new order" text');
		return null;
	}
	
	private function _checkStatusText($input) {
		$tokens = array('{orderid}','{status}','{shopname}');
		$input = str_replace($tokens,'',$input);
		if(!DranerSms_API::isValidText($input))
			return $this->l('Invalid "status alert" text') ;
		return null;
	}
	
	private function _checkStockText($input) {
		$tokens = array('{shopname}','{productname}','{variant}','{productid}','{variantid}','{stock}','{stocklimit}');
		$input = str_replace($tokens,'',$input);
		if(!DranerSms_API::isValidText($input))
			return $this->l('Invalid "stock alert" text');
		return null;
	}
	
	/**
	 * Build administration form's html, using $this->_parameters
	 */
	private function _displayForm() {		
		$this->_html .= '<form action="'.$_SERVER['REQUEST_URI'].'" method="post">';
		$this->_html .= '<script type="text/javascript" src="'.($this->_path).'dranerconfig.js" ></script>';
		$test = $this->_getAdminParameters();		
		foreach ($test as $p) {
			$value = key_exists('name',$p) ? Configuration::get($p['name']) : (key_exists('default', $p) ? $p['default'] : '');
			$type = $p['type'];			
			switch ($type) {
				case 'text':
				case 'password':
				case 'checkbox':
					$extra  = ($type == 'checkbox' && $value == 1) ? ' checked="checked"' : '';
					$extra .= ($type != 'checkbox') ? ' value="'.$value.'"' : '';
					$this->_html .= '<label>'.$p['label'].'</label>';
					$this->_html .= '<div class="margin-form"><input type="'.$type.'" name="'.$p['name'].'"'.$extra.'/>';
					$this->_html .= isset($p['description']) ? '<p>'.$p['description'].'</p></div>' : '</div>';
				break;
				case 'select_multiple':
					$this->_html .= '<label>'.$p['label'].'</label>'
									.'<div class="margin-form">'	
										.'<select name="'.$p['name'].'_select" multiple="multiple" size="'.$p['size'].'">'.$p['options'].'</select>'
										.'<div class="floatr width1">'
											.'<h3 style="color: #000" class="contextnavcenter">'.$this->l('Order states selected to delivery-send').'</h3>'
											.'<p name="'.$p['name'].'_names" ></p>'
											.'<input type="hidden" name="'.$p['name'].'" value="'.$value.'" />'
										.'</div>'
										.'<div>'
											.'<input type="button" name="add_status_send" value="'.$this->l('Add').'"/>'
											.'<input type="button" name="remove_status_send" value="'.$this->l('Remove').'"/>'
											.'<p>'.$p['description'].'</p>'.'</div>'										
									.'</div>';
				break;
				case 'textarea':
					$this->_html .= '<label>'.$p['label'].'</label><div class="margin-form"><textarea rows="2" cols="65" name="'.$p['name'].'" value="'.$value.'">'.$value.'</textarea>';
					$this->_html .= isset($p['description']) ? '<p>'.$p['description'].'</p></div>' : '</div>';
				break;
				
				case 'static':
					$this->_html .= '<label>'.$p['label'].'</label>';
					$this->_html .= '<div class="margin-form" style="color:#000000; font-size:12px; margin-top:3px;">'.$value;
					$this->_html .= isset($p['description']) ? '<p>'.$p['description'].'</p></div>' : '</div>';
				break;
				
				case 'begin_fieldset':
					$this->_html .= '<fieldset><legend>'.$p['label'].'</legend>';
				break;
				
				case 'end_fieldset':
					$this->_html .= '</fieldset><br/>';
				break;
				
				case 'submit':
				default:
					$name = isset($p['submitname']) ? $p['submitname'] : 'DranerSmsupdate';
					$value = isset($p['label']) ? $p['label'] : $this->l('Update settings');
					$this->_html .= '<div class="margin-form"><input class="button" name="'.$name.'" value="'.$value.'" type="submit" /></div>';
				break;
			}
			$this->_html .= '<div style="clear:both; font-size:12px;"></div>';
		}
		$this->_html .= '</form>';	
	}
	
	/* ************** *
	 * HOOK FUNCTIONS *
	 * ************** */
	/**
	 * Hook for new order
	**/
	public function hookNewOrder($params) {	
		if(Configuration::get('DranerSms_ORDER'))
		{
			$order = $params['order'];
			$customer = $params['customer'];
			$currency = $params['currency'];
			
			$this->_sendSMS	(	Configuration::get('DranerSms_ORDER_TEXT'),
								array(	'{price}'	=>	$order->total_paid_real,
										'{currency}'=>	$currency->iso_code,
										'{payment}'	=>	$order->payment,
										'{shopname}'=>	Configuration::get('PS_SHOP_NAME'),
										'{orderid}'	=>	$order->id,
										'{firstname}'=>	$customer->firstname,
										'{lastname}'=>	$customer->lastname
									));					
		}
	}
	
	/**
	 * Hook for update order status
	**/
	public function hookUpdateOrderStatus($params) {
		$newOrderState = $params['newOrderStatus'];
		$idOrder = $params['id_order'];		
		
		$deliveryPhone = null;
						
		if(Configuration::get('DranerSms_STATUS') || Configuration::get('DranerSms_STATUS_DELIVERY_SEND'))
		{
			
			if(Configuration::get('DranerSms_STATUS_DELIVERY_SEND') &&	in_array($newOrderState->id,	json_decode(Configuration::get('DranerSms_STATUS_SELECTED'))))
			{
				$nOrder = new Order($params['id_order']);
				$adress = new Address($nOrder->id_address_delivery);	
				$deliveryPhone = $adress->phone_mobile;		
			}		
						
			if(OrderHistoryCore::getLastOrderState($idOrder))
			{				
				// So this order exists and its status has changed	
				$this->_sendSMS	(Configuration::get('DranerSms_STATUS_TEXT'),
								array(	'{orderid}'	=>	$idOrder,
										'{status}'	=>	$newOrderState->name,
										'{shopname}'=>	Configuration::get('PS_SHOP_NAME')
									),
								$deliveryPhone,
								(!Configuration::get('DranerSms_STATUS')? true : false));				
			}
		}
		
		if(	Configuration::get('DranerSms_LOW_STOCK')
			&& $newOrderState->id != 6
			&& $newOrderState->id != 7
			&& $newOrderState->id != 8	)
		{			
			$order = new OrderCore($idOrder);
			$orderDetails = $order->getProductsDetail();
			$limit = Configuration::get('DranerSms_LOW_STOCK_LIMIT');
			
			foreach ($orderDetails as $detail)
			{
				$currentQty = $detail['product_attribute_id'] != 0 
						? ProductCore::getQuantity($detail['product_id'], $detail['product_attribute_id'])
						: ProductCore::getQuantity($detail['product_id']);
				
				if($currentQty <= $limit)
					$this->_sendSMS	(Configuration::get('DranerSms_LOW_STOCK_TEXT'),
										array(	'{shopname}'	=>	Configuration::get('PS_SHOP_NAME'),
												'{productname}'	=>	$detail['product_name'],
												'{productid}'	=>	$detail['product_id'],
												'{variantid}'	=>	$detail['product_attribute_id'],
												'{stock}'		=>	$currentQty,
												'{stocklimit}'	=>	Configuration::get('DranerSms_LOW_STOCK')
									));			
			}
		}		
	}
		
	public static function getLogs($object, $keyFrom, $indent = '')
	{
		$object = is_object($object) ? (array)$object : $object;
		if (is_array($object))
		{
			error_log($indent.$keyFrom.'=>');
			$indent.='  ';
			foreach ($object as $key => $val)
			{
				if (is_object($val) || is_array($val))
				{
					error_log($indent.$key.'=>');
					self::getLogs($val, $key, $indent.'  ');
				}
				else error_log($indent.$key.'=>'.$val);
			}
		}
		else error_log($indent.$keyFrom.'=>'.$object);
	}
	/* ***************** *
	 * UTILITY FUNCTIONS *
	 * ***************** */
	/**
	 * Returns a list of arrays representing the administration form
	 */
	private function _getAdminParameters() {
		$order_text = $this->l('{firstname} {lastname} has made an order of {price}{currency} on {shopname} using {payment}. Order id : {orderid}');
		$order_text_description = $this->l('Available parameters : ').
				$this->l('price').' {price}, '.$this->l('currency').' {currency}, '.$this->l('payment').' {payment}, '.
				$this->l('shop name').' {shopname}, '.$this->l('order id').' {orderid}, '.
				$this->l('client first name').' {firstname}, '.$this->l('client last name').' {lastname}'
		;
		
		$status_text = $this->l('{shopname} , Order id:  {orderid} , new status :  {status} ');
		$status_text_description = $this->l('Available parameters : ').
				$this->l('shop name').' {shopname}, '.$this->l('order id').' {orderid}, '.$this->l('status').' {status},'.$this->l('shop name').' {shopname}'
		;
		$status_selected = $this->getOptionsFromStatus();
		
		$stock_text = $this->l('{shopname} : stock alert for product {productname} (id:{productid}-{variantid}) : {stock} left');
		$stock_text_description = $this->l('Available parameters : ').
				$this->l('shop name').' {shopname}, '.$this->l('product name').' {productname}, '.
				$this->l('product id').' {productid}, '.$this->l('variant id').'{variantid}, '.
				$this->l('stock left').' {stock}, '.$this->l('stock alert limit').' {stocklimit}'
		;
		
		$user = Configuration::get('DranerSms_USER');
		$password = Configuration::get('DranerSms_PASSWORD');
		
		if($user && $password) {
			$DranerSms_link_description = null;
			$DranerSms_url = 'http://www.123-sms.net/http.php?email='.$user.'&pass='.$password;
		}
		else {
			$DranerSms_link_description = $this->l('Subscribe using this url and get an exclusive offer !');
			$DranerSms_url = 'http://www.123-sms.net/?par=os-comm';
		}
		$DranerSms_link = '<a href="'.$DranerSms_url.'" style="text-decoration:underline;">123-sms.net</a>';
		
		$user_description = $this->l('You need to be registered on ').$DranerSms_link.$this->l(' to use this module.');

		
		$parameters = array(
				array(	'type'	=>'begin_fieldset',	
						'label'	=>$this->l('General parameters')),
				array(	'type'			=>'text', 
						'label'			=>$this->l('123 SMS login'), 				
						'name'			=>'DranerSms_USER',			
						'default'		=>'', 
						'validate'		=>'_checkUser',
						'description'	=>$user_description),				
				array('type'=>'password', 'label'=>$this->l('Password'), 					'name'=>'DranerSms_PASSWORD',		'default'=>'', 'validate'=>'_checkPassword'),
				array('type'=>'text', 'label'=>$this->l('Phone number(s)'),					'name'=>'DranerSms_PHONE',		'default'=>'', 'validate'=>'_checkPhone',
						'description'=>$this->l('You can provide multiple numbers separated by ";"')),
				array('type'=>'submit', 'submitname'=>'DranerSmstest', 'label'=>$this->l('Save and test this configuration')),
				array('type'=>'static', 'label'=>$this->l('Number of SMS sent'),	'name'=>'DranerSms_NUMBER_SENT', 		'default'=>'0'),
				array('type'=>'static', 'label'=>$this->l('Number of failures'),	'name'=>'DranerSms_NUMBER_ERROR', 	'default'=>'0'),
				array('type'=>'static', 'label'=>$this->l('Last failure date'),		'name'=>'DranerSms_LAST_ERROR', 		'default'=>'-'),
				array('type'=>'static', 'label'=>$this->l('SMS Balance'),			'default'=>$this->_getBalance()),
				array('type'=>'end_fieldset'),
	
				array('type'=>'begin_fieldset',	'label'=>$this->l('New order SMS')),
				array('type'=>'checkbox', 'label'=>$this->l('Enabled'), 			'name'=>'DranerSms_ORDER',			'default'=>'0',
						'description'=>$this->l('Check to send a message to the seller when a new order is passed')),
				array('type'=>'textarea', 'label'=>$this->l('Text to be sent'), 	'name'=>'DranerSms_ORDER_TEXT',		'default'=>$order_text,	'validate' => '_checkOrderText',
						'description'=>$order_text_description),
				array('type'=>'submit'),
				array('type'=>'end_fieldset'),
				
				array('type'=>'begin_fieldset',	'label'=>$this->l('status order alert')),
				array('type'=>'checkbox', 'label'=>$this->l('Send to seller phone'), 			'name'=>'DranerSms_STATUS',			'default'=>'0',
						'description'=>$this->l('Check to send a message to the seller when the order status changed')),
				array('type'=>'checkbox', 'label'=>$this->l('Send to delivery adress phone'), 			'name'=>'DranerSms_STATUS_DELIVERY_SEND',			'default'=>'0',
						'description'=>$this->l('Check to send a message to the delivery adress phone when the order status changed (no send if no mobile phone')),
				array(	'type'			=>'select_multiple',
						'label'			=>$this->l('States changes to notify'),
						'name'			=>'DranerSms_STATUS_SELECTED',
						'description'	=>$this->l('Only status selected will be notificated to delivery phone'),
						'size'			=>6,
						'options'		=>$this->getOptionsFromStatus(),
						'default'		=>'[]'),
				array('type'=>'textarea', 'label'=>$this->l('Text to be sent'), 	'name'=>'DranerSms_STATUS_TEXT',	'default'=>$status_text, 'validate' => '_checkStatusText',
						'description'=>$status_text_description),
				array('type'=>'submit'),
				array('type'=>'end_fieldset'),
				
				array('type'=>'begin_fieldset',	'label'=>$this->l('Stock alert SMS')),
				array('type'=>'checkbox', 	'label'=>$this->l('Enabled'), 			'name'=>'DranerSms_LOW_STOCK',			'default'=>'0',
						'description'=>$this->l('Send a message when the stock of a product gets below the limit')),
				array('type'=>'text', 		'label'=>$this->l('Low stock limit'), 	'name'=>'DranerSms_LOW_STOCK_LIMIT',      'default'=>'10',
						'description'=>$this->l('The message will be sent when the stock falls to this limit (0 = warning for out of stock)')),
				array('type'=>'textarea', 	'label'=>$this->l('Text to be sent'), 	'name'=>'DranerSms_LOW_STOCK_TEXT',		'default'=>$stock_text,	'validate' => '_checkStockText',
						'description'=>$stock_text_description),
				array('type'=>'submit'),
				array('type'=>'end_fieldset'),
	
				array('type'=>'begin_fieldset',	'label'=>$this->l('About 123 SMS')),
				array('type'=>'static', 	'label'=>$this->l('Developed by'), 					'default'=>'123 SMS'),
				array('type'=>'static', 	'label'=>$this->l('Version'), 						'default'=>sprintf('%01.1f',$this->version)),
				array('type'=>'static', 	'label'=>$this->l('Website to buy SMS packages'),	'default'=>$DranerSms_link,
						'description'=>$DranerSms_link_description),
				array('type'=>'end_fieldset')
		);
		return $parameters;
	}
	
	private function _sendSMS($textTemplate, $arrayContent, $deliveryPhone = null, $deliveryOnly = false) {
		$login = Configuration::get('DranerSms_USER');
		$password = Configuration::get('DranerSms_PASSWORD');
		$phones = explode	(';',(!$deliveryOnly 	?	Configuration::get('DranerSms_PHONE') : '')
								.($deliveryPhone 	? ';'.$deliveryPhone : '')
							);		
		$text = DranerSms_API::prepareText($textTemplate,$arrayContent);
				
		$return = true;
		if($text)
		{
			foreach($phones as $phone)
			{
				$phone = preg_replace('#[^0-9]#','',$phone);	//Remove non-digit characters
				$api = new DranerSms_API($login,$password,$text);				
				$result = $api->sendSMS($phone);
			}
		}else 	$return = false;
		
		return $return;
	}
	
	private function getOptionsFromStatus()
	{
		$statusArray = OrderStateCore::getOrderStates(ConfigurationCore::get('PS_LANG_DEFAULT'));
		$htmlOptions = '';
				
		foreach ($statusArray as $status)
		{
			$htmlOptions .= '<option value="'.$status['id_order_state'].'">'.$status['name'].'</option>';
		}
		
		return $htmlOptions;
	}
	
	private function _getBalance() {
		if($this->_balance !== null && $this->_balance !== "-") {
			return $this->_balance;
		}
		
		$login = Configuration::get('DranerSms_USER');
		$password = Configuration::get('DranerSms_PASSWORD');
		if($login && $password) {
			
		// url d'accès à la passerelle
		$url = "http://123-sms.net/solde_comptes.php?email=$login&pass=$password";
		// connexion
		$reponse = file_get_contents($url);
		// affichage de la réponse

			if($reponse != "0")
				$this->_balance = $reponse;
			else
				$this->_balance = $this->l('Negative _ You must credit your account');
		}
		else {
			$this->_balance = "-";
		}
		
		return $this->_balance;
	}
}
?>
